/*
 * $Id: externalform.c,v 1.36 2003/07/05 20:46:37 prussar Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "debug.h"
#include "document.h"
#include "genericfile.h"
#include "jogdial.h"
#include "resourceids.h"
#include "uncompress.h"
#include "util.h"

#include "externalform.h"


/***********************************************************************
 *
 *      Local functions
 *
 ***********************************************************************/
static void ExternalLinksFormInit( void ) EXTERNALFORM_SECTION;


/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static Int16 linkIdx;



/* Set current link in the ExternalLinks record */
void SetLinkIndex
    (
    Int16 index /* index of record */
    )
{
    linkIdx = index;
}



/* Initialize field with URL string */
Boolean AddURLToField
    (
    FieldType*  fldPtr, /* pointer to field */
    Int16       index   /* index in URL record */
    )
{
    MemHandle   uncompressHandle;
    Header*     linkDocument;
    Int16       count;
    Int16       offset;
    Int16*      numP;
    MemHandle   linkHandle;
    Char*       text;

    linkHandle  = NULL;
    text        = NULL;
    count       = index;
    offset      = 0;

    if ( PLUCKER_LINKS_ID != 0 ) {
        linkHandle = ReturnRecordHandle( PLUCKER_LINKS_ID );
        if ( linkHandle != NULL ) {
            linkDocument = MemHandleLock( linkHandle );
            numP = (Int16*) ( linkDocument + 1 );
            while ( *numP < index ) {
                offset  = *numP;
                numP   += 2;
            }
            numP    += 1;
            count   -= offset;
            MemHandleUnlock( linkHandle );
            FreeRecordHandle( &linkHandle );
    
            linkHandle = ReturnRecordHandle( *numP );
            if ( linkHandle == NULL )
                return false;
    
            uncompressHandle    = NULL;
            linkDocument        = MemHandleLock( linkHandle );
            if ( linkDocument->type == DATATYPE_LINKS_COMPRESSED ) {
                ErrTry {
                    uncompressHandle = Uncompress( linkDocument );
                }
                ErrCatch( UNUSED_PARAM( err ) ) {
                    uncompressHandle = NULL;
                } ErrEndCatch
                MemHandleUnlock( linkHandle );
                FreeRecordHandle( &linkHandle );
                if ( uncompressHandle )
                    text = MemHandleLock( uncompressHandle );
            }
            else if ( linkDocument->type == DATATYPE_LINKS ) {
                text = (Char*) ( linkDocument + 1 );
            }
    
            if ( text != NULL ) {
                while ( --count )
                    text += StrLen( text ) + 1;
                FldInsert( fldPtr, text, StrLen( text ) );
            }
            if ( linkHandle != NULL ) {
                MemHandleUnlock( linkHandle );
                FreeRecordHandle( &linkHandle );
            }
            if ( uncompressHandle != NULL )
                MemHandleUnlock( uncompressHandle );
        }
    }
    if ( text == NULL ) {
        Char buff[ 100 ];

        SysCopyStringResource( buff, strExternNoURL );
        FldInsert( fldPtr, buff, StrLen( buff ) );
        return false;
    }
    return true;
}



/* Initialize the externallinks form */
static void ExternalLinksFormInit( void )
{
    FormType*   externallinksForm;
    FieldType*  field;

    field = GetObjectPtr( frmExternalLinksLink );
    externallinksForm = FrmGetFormPtr( frmExternalLinks );
    if ( AddURLToField( field, linkIdx ) )
        FrmShowObject( externallinksForm, FrmGetObjectIndex( externallinksForm,
                                            frmExternalLinksCopy) );
    else
        FrmHideObject( externallinksForm, FrmGetObjectIndex( externallinksForm,
                                            frmExternalLinksCopy) );
    FrmDrawForm( externallinksForm );
}



/* Write the text from a TextField to a Memo */
void WriteMemo
    (
    FieldType* field    /* field to get text from */
    )
{
    static UInt16 index     = dmMaxRecordIndex;
    static UInt32 offset    = 0;
    static UInt32 secs      = 0;

    UInt16 length;

    length = FldGetTextLength( field );
    if ( length == 0 )
        return;

    if ( secs == 0 ) {
        DateTimeType dateTime;

        secs = TimGetSeconds();
        TimSecondsToDateTime( secs, &dateTime );
        WriteMemoEntry( &index, &offset, "%s %4d-%02d-%02d %02d:%02d\n",
            "Plucker URLs", dateTime.year, dateTime.month, dateTime.day,
            dateTime.hour, dateTime.minute );
    }
    WriteMemoEntry( &index, &offset, "%s\n", FldGetTextPtr( field ) );
}



/* Event handler for the externallinks form */
Boolean ExternalLinksFormHandleEvent
    (
    EventType* event  /* pointer to an EventType structure */
    )
{
    Boolean     handled;
    FieldType*  field;

    handled = false;
    field   = GetObjectPtr( frmExternalLinksLink );

    switch ( event->eType ) {
        case ctlSelectEvent:
        case keyDownEvent:
            if ( event->data.ctlEnter.controlID == frmExternalLinksCopy
                 || IsJogdialRelease( event->data.keyDown.chr )
               )
                WriteMemo( field );
            else if ( event->data.ctlEnter.controlID != frmExternalLinksBack
                      && ! IsJogdialBack( event->data.keyDown.chr )
                    )
                break;

            FrmGotoForm( GetMainFormId() );
            handled = true;
            break;

        case frmOpenEvent:
            ExternalLinksFormInit();
            handled = true;
            break;

        case frmCloseEvent:
            handled = false;
            break;

        default:
            handled = false;
    }

    return handled;
}

